#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>

#include <QToolButton>
#include <QTreeWidgetItem>
#include <QTableWidgetItem>

#include <QXmlStreamWriter>

#include <QGraphicsScene>
#include <QAbstractGraphicsShapeItem>
#include <QGraphicsRectItem>
#include <QGraphicsEllipseItem>

#include <QLinearGradient>
#include <QRadialGradient>

namespace Ui {
class MainWindow;
}

class Rectangle : public QObject, public QGraphicsRectItem
{
Q_OBJECT
protected:
    void dragEnterEvent (QGraphicsSceneDragDropEvent *event);
    void dropEvent (QGraphicsSceneDragDropEvent *event);
public:
    Rectangle ();
public slots:
    void move (int x, int y) { moveBy (x, y); }
    void flash ()
    {
        QLinearGradient g (0, 0, 100, 100);
        g.setColorAt(0, QColor ("blue"));
        g.setColorAt(1, QColor ("red"));
        QBrush brush (g);
        setBrush (brush);
    }
    //void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
};

class Ellipse : public QObject, public QGraphicsEllipseItem
{
   Q_OBJECT
protected:
    void dragEnterEvent (QGraphicsSceneDragDropEvent * event);
    void dropEvent (QGraphicsSceneDragDropEvent * event);
public:
    Ellipse ();
public slots:
    void move (int x, int y) { moveBy (x, y); }
    void flash ()
    {
        QRadialGradient g (50, 50, 50, 50, 50);
        g.setColorAt(0, QColor ("blue"));
        g.setColorAt(1, QColor ("yellow"));
        QBrush brush (g);
        setBrush (brush);
        setRect(0, 0, 100, 100);
    }
    void alt (int w, int h)
    {
        QRadialGradient g (w, h, w, w, h);
        g.setColorAt(0, QColor ("blue"));
        g.setColorAt(1, QColor ("yellow"));
        QBrush brush (g);
        setBrush (brush);
        setRect(0, 0, 2*w, 2*h);
    }
};

class ParamList : public QMap <QString, QString>
{
   public:
      int number (QString key, int default_value = 0);
};

class TreeNode : public QTreeWidgetItem
{
public :
    QGraphicsItem * elem;
    TreeNode () : elem (NULL) { }
};

class ColorButton : public QToolButton
{
private :
    QString name;
    QColor color;
public :
    ColorButton (QString s);
protected :
    void mousePressEvent(QMouseEvent *event);
private :
    QPixmap createPixmap (bool white = false);
};

class ShapeButton : public QToolButton
{
private :
    QString name;
    QString code;
public :
    ShapeButton (QString n, QString c);
protected :
    void mousePressEvent(QMouseEvent *event);
};

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

public:
    void addGraphicsItem (QGraphicsItem * shape, QString code, bool plus, int relx = 0, int rely = 0);

private:
    void readFile (QString fileName);
    void writeFile (QString fileName);
    void writeItem (QXmlStreamWriter &writer, QTreeWidgetItem *item);

    void showTable (QGraphicsItem * elem);

private slots:
    void on_treeWidget_itemClicked (QTreeWidgetItem *item, int column);

    void on_tableWidget_itemChanged (QTableWidgetItem *item);

    void on_actionOpen_triggered();
    void on_actionSave_triggered();
    void on_actionQuit_triggered();

    void on_actionCopy_triggered();
    void on_actionPaste_triggered();
    void on_actionRun_triggered();

    void scene_selection_changed ();

    void on_graphicsView_customContextMenuRequested(const QPoint &pos);

private:
    Ui::MainWindow * ui;

    QGraphicsScene * scene;
    QGraphicsItem * current_element;
};

#endif // MAINWINDOW_H
